@import '~@synerty/peek-theme/scss/_mixins.web';
@import '~@synerty/peek-theme/scss/_variables.web';

.peek-tabs-section {
  width: unset;
}


.modal-open.modal{
  overflow-y: unset;
}

.modal-body {
  overflow-y: auto;
  max-height: 85vh;
}

.edit-create-branch-dialog {
  padding: $general_padding_1;
  top: $title_bar_height;
}

/*
.modal-backdrop {
  float: left;
  top: 0px;
  bottom:0px;
  left:0px;
  right:0px;
  background-color: $primary_dark;
}
*/